import {EntityRepository, getConnection, Repository} from "typeorm";
import {RetailerService} from "./entities/retailer-service.entity";

@EntityRepository(RetailerService)
export class RetailerServiceRepository extends Repository<RetailerService> {
    async createEntity(dto: any): Promise<RetailerService | Error> {
        try {
            const {
                retailerId,
                retailerName,
                retailerImage,
                retailerAddress,
                retailerDeliverySettings,
                accessCount,
                retailerMobileNumber,
                alpineIqApiKey,
                alpineIqUID,
                hours
            } = dto;
            const entity = this.create({
                retailerId,
                retailerName,
                retailerImage,
                retailerAddress,
                retailerDeliverySettings,
                accessCount,
                retailerMobileNumber,
                alpineIqApiKey,
                alpineIqUID,
                hours
            });
            await this.save(entity);
            return entity;
        } catch (e) {
            return e;
        }
    }

    async findOneEntity(id: string): Promise<RetailerService | Error> {
        try {
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async findByFilterQuery(query: any): Promise<any | Error> {
        try {
            return await this.find({
                where: {
                    ...query,
                    isDeleted: false
                }
            });
        } catch (e) {
            return e;
        }
    }

    async topFiveRetailerEntity(): Promise<any | Error> {
        try {
            return await this.find({
                where: {
                    isDeleted: false
                },
                order: {
                    accessCount: "DESC"
                },
                take: 5
            });
        } catch (e) {
            return e;
        }
    }

    async deleteEntity(id: string): Promise<boolean | Error> {
        try {
            const updateResult = await getConnection()
                .createQueryBuilder()
                .update(RetailerService)
                .set({isDeleted: true})
                .where("id = :id", {id: id})
                .execute();
            if (updateResult.affected > 0) {
                return true;
            }
            return false;
        } catch (e) {
            return e;
        }
    }
}