import {EntityRepository, getConnection, Repository} from "typeorm";
import {Newsletter} from "./entities/newsletter.entity";
import {CreateNewsletterDto} from "./dto/create-newsletter.dto";
import {UpdateNewsletterDto} from "./dto/update-newsletter.dto";

@EntityRepository(Newsletter)
export class NewsletterRepository extends Repository<Newsletter> {
    async createEntity(dto: CreateNewsletterDto): Promise<Newsletter | Error> {
        try {
            const {email} = dto;
            const entity = this.create({
                email
            });
            await this.save(entity);
            return entity;
        } catch (e) {
            return e;
        }
    }

    async findOneEntity(id: string): Promise<Newsletter | Error> {
        try {
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async updateEntity(id: string, dto: UpdateNewsletterDto): Promise<Newsletter | Error> {
        try {
            await this.update(id, {
                email: dto.email
            });
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async findByFilterQuery(query: any): Promise<Newsletter[] | Error> {
        try {
            return await this.find({
                where: {...query, isDeleted: false}
            });
        } catch (e) {
            return e;
        }
    }

    async deleteEntity(id: string): Promise<boolean | Error> {
        try {
            const updateResult = await getConnection()
                .createQueryBuilder()
                .update(Newsletter)
                .set({isDeleted: true})
                .where("id = :id", {id: id})
                .execute();
            if (updateResult.affected > 0) {
                return true;
            }
            return false;
        } catch (e) {
            return e;
        }
    }

    async countAllQuery(): Promise<any | Error> {
        try {
            return await this.count({
                where: {
                    isDeleted: false
                }
            });
        } catch (e) {
            return e;
        }
    }
}